package org.h2;

import java.io.File;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.h2.tools.Server;

/**
 * @author gaellalire
 */
public class H2VestigeLauncher implements Runnable {

    private static final Logger LOGGER = Logger.getLogger(H2VestigeLauncher.class.getName());

    private static boolean ncl = true;

    final Server server;

    public H2VestigeLauncher(final File home) throws SQLException {
        LOGGER.info("CONSTRUCT NEW OBJECT");
        server = Server.createTcpServer("-baseDir", home.getPath());
    }

    private volatile boolean started = false;

    public void run() {
        LOGGER.info("NEW CLASSLOADER " + ncl);
        ncl = false;

        Thread launcherThread = new Thread("h2TCPServer") {
            @Override
            public void run() {
                try {
                    server.start();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
                started = true;
            }
        };
        launcherThread.start();
        try {
            synchronized (this) {
                wait();
            }
        } catch (InterruptedException e) {
            while (true) {
                try {
                    launcherThread.join();
                    break;
                } catch (InterruptedException e1) {
                    LOGGER.fine("Ignore interrupt");
                }
            }
            if (started) {
                server.stop();
            }
            Thread currentThread = Thread.currentThread();
            ThreadGroup threadGroup = currentThread.getThreadGroup();
            int activeCount = threadGroup.activeCount();
            while (activeCount != 1) {
                Thread[] list = new Thread[activeCount];
                int enumerate = threadGroup.enumerate(list);
                for (int i = 0; i < enumerate; i++) {
                    Thread t = list[i];
                    if (t == currentThread) {
                        continue;
                    }
                    t.interrupt();
                }
                for (int i = 0; i < enumerate; i++) {
                    Thread t = list[i];
                    if (t == currentThread) {
                        continue;
                    }
                    try {
                        t.join();
                    } catch (InterruptedException e1) {
                        LOGGER.log(Level.FINE, "Ignore interrupt", e1);
                    }
                }
                activeCount = threadGroup.activeCount();
            }
        } finally {
            started = false;
        }
    }

}
